const axios = require('axios');
const openApiConfig = require('./config')
const https = require('https')
const { v4: uuidv4 } = require('uuid')
const crypto = require('crypto')
const NodeRSA = require('node-rsa')

// 缓存公钥
let pubKey = ''
let pubKeyExp = ''
let antiReplayRand = ''

process.on('uncaughtException', err => {
    console.log(err)
})

const instance = axios.create({
    baseURL: openApiConfig.CONSOLE_ADDRESS,
    timeout: 10000,
    httpsAgent: new https.Agent({
        rejectUnauthorized: false
    })
})

instance.interceptors.request.use(config => {
    const urlString = `${config.baseURL}${config.url}`
    // 如果接口需要签名校验，则添加签名请求头
    if (config.authorize) {
        let customHeaders = {}
        if (config.method === 'get') {
            customHeaders = prepareAuthHeaders(urlString, config.params)
            for (const key in customHeaders) {
                config.headers[key] = customHeaders[key]
            }
        }
        if (config.method === 'post') {
            customHeaders = prepareAuthHeaders(urlString, null, config.data)
            for (const key in customHeaders) {
                config.headers[key] = customHeaders[key]
            }
        }
    }
    return config
}, error => {
    return Promise.reject(error);
})

const axiosGet = (path, query, authorize = true) => {
    if (!query) {
        query = {}
    }
    return instance({
        method: 'get',
        url: `${path}`,
        authorize,
        params: query
    })
}

const axiosPost = (path, data, authorize = true) => {
    return instance({
        method: 'post',
        url: `${path}`,
        data,
        authorize
    })
}

const getConsoleConfig = () => {
    const path = '/api/v1/admin/getConfig'
    return new Promise((resolve, reject) => {
        axiosGet(path, null, false).then(res => {
            if (res.status === 200) {
                resolve(res.data)
            } else {
                reject(res)
            }
        }).then(err => {
            reject(err)
        })
    })
}

const encodeRSA = (content, modulus, exponent) => {
    try {
        const mod = Buffer.from(modulus, 'hex')
        const exp = Buffer.alloc(4)
        exp.writeInt32BE(Number(exponent), 0)

        const key = new NodeRSA();
        key.setOptions({ encryptionScheme: 'pkcs1' })
        key.keyPair.setPublic(mod, exp)

        return key.encrypt(content, 'hex')
    } catch (e) {
        console.log(e);
        return ''
    }
}

const encrypt = async password => {
    // 从控制台获取公钥信息
    const consoleConfig = await getConsoleConfig();
    if (!pubKey || !pubKeyExp || !antiReplayRand) {
        pubKey = consoleConfig.data.crypto.pubKey
        pubKeyExp = consoleConfig.data.crypto.pubKeyExp
        antiReplayRand = consoleConfig.data.crypto.antiReplayRand
    }


    // 拼接密码和防重放随机数
    const message = password + '_' + antiReplayRand;

    // 进行加密
    const encrypted = encodeRSA(message, pubKey, pubKeyExp)

    // 返回加密后的十六进制字符串
    return encrypted;
}

function calSignature(key, urlString, query = null, body = null) {
    // 从url中解析path路径
    const parsedUrl = new URL(urlString);
    const path = parsedUrl.pathname;

    // 按规则拼接query参数的kv_str("k=v")
    let paramsList = [];
    if (query !== null) {
        const sortedQueryKeys = Object.keys(query).sort();
        sortedQueryKeys.forEach((key) => {
            paramsList.push(`${key}=${query[key]}`);
        });
    }

    // 准备body参数，严格json字符串
    if (body !== null) {
        paramsList.push(JSON.stringify(body, null, 0));
    }

    // 拼接签名字符串
    let strToSign = path;
    if (paramsList.length) {
        strToSign += '?' + paramsList.join('&');
    }

    // 签名
    const hmac = crypto.createHmac('sha256', key);
    hmac.update(strToSign);
    return hmac.digest('hex');
}

function prepareAuthHeaders(urlString, query = null, body = null) {
    const appId = openApiConfig.API_ID;
    const appSecret = openApiConfig.API_SECRET;
    // 获取十位时间戳
    const timestamp = Math.floor(Date.now() / 1000).toString();

    // 使用uuid生成随机数
    const nonce = uuidv4();

    // 按照规则拼接签名密钥
    const key = `appId=${appId}&appSecret=${appSecret}&timestamp=${timestamp}&nonce=${nonce}`;
    const sign = calSignature(key, urlString, query, body);

    return {
        'x-ca-key': appId,
        'x-ca-timestamp': timestamp,
        'x-ca-nonce': nonce,
        'x-ca-sign': sign,
    };
}

module.exports = {
    instance,
    axiosGet,
    axiosPost,
    getConsoleConfig,
    encrypt
}